@@ -1,3 +1,4 @@
+import logging
 import math
 from inspect import signature
 from pathlib import Path
@@ -8,7 +9,8 @@ from feeluown.excs import NoUserLoggedIn, MediaNotFound
 from feeluown.library import AbstractProvider, ProviderV2, ProviderFlags as Pf, UserModel, VideoModel, \
     BriefPlaylistModel, BriefSongModel, LyricModel, SupportsSongSimilar, BriefSongProtocol, SupportsSongHotComments, \
     SupportsAlbumGet, BriefAlbumModel, SupportsPlaylistAddSong, SupportsPlaylistRemoveSong, BriefUserModel, \
-    BriefArtistModel, SearchType as FuoSearchType, ModelType, SimpleSearchResult
+    BriefArtistModel, SearchType as FuoSearchType, ModelType, SimpleSearchResult, ModelNotFound, Collection, \
+    CollectionType
 from feeluown.media import Quality, Media, MediaType, VideoAudioManifest
 from feeluown.utils.reader import SequentialReader
 
@@ -30,9 +32,12 @@ from fuo_bilibili.danmaku2ass import Danmaku2ASS
 from fuo_bilibili.model import BSearchModel, BSongModel, BPlaylistModel, BArtistModel, BCommentModel, BVideoModel, \
     BAlbumModel
 from fuo_bilibili.util import json_to_lrc_text
+from fuo_bilibili.login import load_user_cookies
 
+logger = logging.getLogger(__name__)
 SEARCH_TYPE_MAP = {
-    FuoSearchType.vi: BilibiliSearchType.LIVE_ROOM,  # 对应直播间
+    # 对应直播间
+    FuoSearchType.vi: BilibiliSearchType.LIVE_ROOM,
     FuoSearchType.ar: BilibiliSearchType.BILI_USER,  # 对应B站用户
     FuoSearchType.so: BilibiliSearchType.VIDEO,  # 对应投稿视频
     FuoSearchType.al: (BilibiliSearchType.MEDIA, BilibiliSearchType.BANGUMI),  # 对应番剧电影
@@ -100,7 +105,7 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
             ModelType.album: (Pf.model_v2 | Pf.get),
         }
 
-    def __init__(self):
+    def __init__(self, enable_live_room_as_video=True):
         super(BilibiliProvider, self).__init__()
         self._api = BilibiliApi()
         self._user = None
@@ -108,8 +113,13 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
         self._video_cids = dict()
         self._video_avids = dict()
 
+        self.enable_live_room_as_video = enable_live_room_as_video
+
     def _format_search_request(self, keyword, type_) -> Union[SearchRequest, Tuple[SearchRequest]]:
-        btype = SEARCH_TYPE_MAP.get(type_)
+        if self.enable_live_room_as_video is False and type_ == FuoSearchType.vi:
+            btype = BilibiliSearchType.VIDEO
+        else:
+            btype = SEARCH_TYPE_MAP.get(type_)
         if btype is None:
             raise NotImplementedError
         if isinstance(btype, Tuple):
@@ -124,17 +134,25 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
     def request_key(self) -> RequestLoginKeyResponse:
         return self._api.request_login_key()
 
-    def cookie_check(self):
-        return self._api.cookie_check()
+    def auto_login(self):
+        logger.info("Try auto logging...")
+        cookiedict = load_user_cookies()
+        if cookiedict:
+            logger.info("Try fetch user info with cookies...")
+            self._api.from_cookiedict(cookiedict)
+            try:
+                user = self.user_info()
+            except RuntimeError as e:
+                logger.warning(f'Auto login failed: {e}')
+            else:
+                self.auth(user)
+                logger.info(f'Auto login success: {user.name}')
+        else:
+            logger.info('No cookies found')
 
-    def auth(self, _):
-        self._api.load_cookies()
-        try:
-            self._user = self.user_info()
-        except RuntimeError as re:
-            self._api.remove_cookie()
-            print(str(re) + ' 请重新登录')
-        return self._user
+    def auth(self, user):
+        self._api.enable_auto_save_cookie = bool(user)
+        self._user = user
 
     def has_current_user(self) -> bool:
         return self._user is not None
@@ -146,6 +164,7 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
 
     def user_info(self) -> UserModel:
         data: NavInfoResponse.NavInfoResponseData = self._api.nav_info().data
+        self._api.set_wbi(data.wbi_img)
         user = UserModel(
             source=__identifier__,
             identifier=str(data.mid),
@@ -167,6 +186,8 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
         return self._api.sms_code_login(request)
 
     def search(self, keyword, type_, *args, **kwargs) -> Optional[SimpleSearchResult]:
+        if not self.has_current_user():
+            return None
         request = self._format_search_request(keyword, type_)
         if isinstance(request, Tuple):
             response = tuple([self._api.search(r) for r in request])
@@ -187,7 +208,7 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
         song = BSongModel.create_info_model(response)
         if page_num is not None:
             try:
-                song.children[int(page_num)-1]
+                song.children[int(page_num) - 1]
             except IndexError:
                 return None
         return song
@@ -255,7 +276,7 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
         if cid is None:
             info = self._api.video_get_info(VideoInfoRequest(bvid=bvid))
             if page_num >= 0:
-                cid = info.data.pages[page_num-1].cid
+                cid = info.data.pages[page_num - 1].cid
             else:
                 if len(info.data.pages) > 1 and page_num != -1:
                     raise MediaNotFound(reason=MediaNotFound.Reason.check_children)
@@ -342,7 +363,6 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
         if audios is None or len(audios) == 0:
             return None
         # select video
-        print(response.data.dash.video)
         max_quality_code = VideoQualityNum.get_max_from_quality(quality)
         select_quality = max(filter(lambda c: c.value <= max_quality_code, self._video_quality_codes[video.identifier]),
                              key=lambda c: c.value)
@@ -380,6 +400,8 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
                 qualities.append(Quality.Audio.sq)
                 continue
             qualities.append(Quality.Audio.hq)
+        if response.data.dash.flac is not None and response.data.dash.flac['audio'] is not None:
+            qualities.append(Quality.Audio.shq)
         print(list(set(qualities)))
         return list(set(qualities))
 
@@ -398,6 +420,11 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
             cid=cid,
             fnval=VideoFnval.DASH
         ))
+        if Quality.Audio.shq == quality:
+            if response.data.dash.flac is not None and response.data.dash.flac['audio'] is not None:
+                return Media(response.data.dash.flac['audio']['base_url'], type_=MediaType.audio, format='m4s',
+                             bitrate=int(response.data.dash.flac['audio']['bandwidth'] / 1000),
+                             http_headers={'Referer': 'https://www.bilibili.com/'})
         audios = sorted(response.data.dash.audio, key=lambda a: a.bandwidth, reverse=True)
         selects: Optional[List[PlayUrlResponse.PlayUrlResponseData.Dash.DashItem]] = None
         match quality:
@@ -431,6 +458,18 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
             name=user.uname,
         ) for user in resp.data.list]
 
+    def current_user_list_playlists(self):
+        p1 = self.special_playlists()
+        p2 = self.user_playlists(self.get_current_user().identifier)
+        p3 = self.audio_favorite_playlists()
+        p4 = self.audio_collected_playlists()
+        return p1 + p2 + p3 + p4
+
+    def current_user_fav_create_playlists_rd(self):
+        p1 = self.fav_playlists(self.get_current_user().identifier)
+        p2 = self.audio_collected_playlists()
+        return p1 + p2
+
     def audio_favorite_playlists(self) -> List[BriefPlaylistModel]:
         resp = self._api.audio_favorite_list(PaginatedRequest(ps=100, pn=1))
         return BPlaylistModel.create_audio_model_list(resp)
@@ -477,7 +516,7 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
             _, __, ssid = identifier.split('_')
             resp = self._api.media_bangumi_get_list(MediaGetListRequest(season_id=ssid))
             return BAlbumModel.create_season_model(resp.result)
-        return None
+        raise ModelNotFound(f'album:{identifier} not found')
 
     def playlist_get(self, identifier: str) -> Optional[BPlaylistModel]:
         # fixme: fuo should support playlist_get v2 first
@@ -623,6 +662,18 @@ class BilibiliProvider(AbstractProvider, ProviderV2, SupportsSongSimilar, Suppor
 
         return SequentialReader(g(), total)
 
+    def rec_a_collection_of_videos(self):
+        if self.has_current_user():
+            resp = self._api.home_recommend_videos(HomeRecommendVideosRequest(ps=12))
+            videos = [BVideoModel.create_recommend_video_model(v) for v in resp.data.item]
+        else:
+            videos = []
+        return Collection(
+            name='推荐视频',
+            type_=CollectionType.only_videos,
+            models=videos,
+        )
+
     @property
     def identifier(self):
         return __identifier__
